#include "StdAfx.h"
#include "GameTalkAPIGlobal.h"
#include "WASListMemberGroup.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASListMemberGroup::CWASListMemberGroup(CWASActor *pActor) : 
	CWASCommand(pActor)
{
    m_nGroupType = 0;
}


CWASListMemberGroup::~CWASListMemberGroup(void)
{
}


void CWASListMemberGroup::ExecuteAsync()
{
	m_listMemberGroupPrivate.clear();
	m_listMemberGroup.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL;
	if( this->m_nGroupType == 1 )
	{
		strURL = m_pActor->GetWASServerAddress();
		strURL += _T("/ne/org/ListOrgMemberGroup.php");
	}
	else
	{
		strURL = m_pActor->GetWASServerAddress();
		strURL += _T("/ne/org/ListWorkMemberGroup.php");
	}

	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().AddBodyVariable("tenant", W2U8(m_wsTenant.c_str()));
	m_httpFacade.session().AddBodyVariable("langcode", W2U8(m_wsLangCode.c_str()));

	if( this->m_nGroupType == 1 )
		m_httpFacade.session().AddBodyVariable("custco_id", W2U8(m_wsCustcoID.c_str()));

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}



bool CWASListMemberGroup::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	//
	//	[XML FORMAT]
	/*
		<?xml version="1.0" encoding="UTF-8" ?>
		<nphone>
			<nodes num='#ڿ'>
				<node grprid='#ڿ'  grpname='#ڿ' />
			</nodes>
		</nphone>
	*/
	if(pDom == NULL)
		return false;

	nfc::IIkXMLNode* pNode = pDom->GetdocumentElement()->selectSingleNode("nodes");
	if (pNode == NULL)
	{
		m_strWasErrMsg += _T("can not find \"<nodes>\" in group list.");
		return false;
	}

	nfc::IIkXMLNodeList *pList = pNode->selectNodes("node");		
	nfc::IIkXMLAttribute* pAttr;
	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	m_listMemberGroupPrivate.clear();


	for (int i = 0; i < pList->Getlength(); i++)
	{	
		GT_ORG_GROUP_DATA group;
		pNode = pList->Getitem(i);
		GT_ASSERT(pNode);

		if (NULL == (pAttr = pNode->getAttribute("grpid")))
		{
			continue;
		}
		if (NULL != (pAttr = pNode->getAttribute("grpid")))
		{
			group.strGroupID=U82W(pAttr->Gettext().c_str());
		}

		if (NULL != (pAttr = pNode->getAttribute("grpname")))
		{
			group.strGroupName = U82W(pAttr->Gettext().c_str());
		}
		if (NULL != (pAttr = pNode->getAttribute("upr_dept_id")))
		{
			group.strParentGroupID = U82W(pAttr->Gettext().c_str());
		}
		m_listMemberGroupPrivate.push_back(group);
	}

	return true;
}


// override in drived class
void CWASListMemberGroup::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	// WSTRING LPCWSTR ȯؼ 迭  ־!!!
	for(size_t i = 0; i < m_listMemberGroupPrivate.size(); i++)
	{
		GT_ORG_GROUP_DATA sTemp;
		sTemp = m_listMemberGroupPrivate[i];
		m_listMemberGroup.push_back(sTemp);
	}

	GT_ORG_GROUP_DATA_LIST sList;
	sList.nSize = m_listMemberGroup.size();
	sList.pList = sList.nSize > 0 ? &m_listMemberGroup[0] : NULL;
	
	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		pObserver->OnGTWASActor_ResponseMemberGroupList(m_pActor, GetID(), m_eError, sList);
	}

	//LeaveCriticalSection(pCS);
	
	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASListMemberGroup::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	// WSTRING LPCWSTR ȯؼ 迭  ־!!!
	for(size_t i = 0; i < m_listMemberGroupPrivate.size(); i++)
	{
		GT_ORG_GROUP_DATA sTemp;
		sTemp = m_listMemberGroupPrivate[i];
		m_listMemberGroup.push_back(sTemp);
	}

	GT_ORG_GROUP_DATA_LIST sList;
	sList.nSize = m_listMemberGroup.size();
	sList.pList = sList.nSize > 0 ? &m_listMemberGroup[0] : NULL;
	
	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		pObserver->OnGTWASActor_ResponseMemberGroupList(m_pActor, GetID(), m_eError, sList);
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
